# AUTOGENERATED! DO NOT EDIT! File to edit: notebooks/custom_tf_training.ipynb (unless otherwise specified).

__all__ = []

# Cell
import os
from nbdev.showdoc import *
from fastcore.test import *


import tensorflow as tf
import sklearn
import numpy as np
from sklearn.datasets import fetch_california_housing
from sklearn.model_selection import train_test_split
from sklearn.preprocessing import StandardScaler
import matplotlib as mpl
import matplotlib.pyplot as plt
mpl.rc('axes', labelsize=14)
mpl.rc('xtick', labelsize=12)
mpl.rc('ytick', labelsize=12)


test_eq(sklearn.__version__ > "0.20", True)
test_eq(tf.__version__ > "2.0.0", True)