# AUTOGENERATED BY NBDEV! DO NOT EDIT!

__all__ = ["index", "modules", "custom_doc_links", "git_url"]

index = {"generate_1d_data": "custom_pytorch_training.ipynb",
         "generate_2d_data": "custom_pytorch_training.ipynb",
         "train": "custom_pytorch_training.ipynb",
         "eval_loss": "custom_pytorch_training.ipynb",
         "train_epochs": "custom_pytorch_training.ipynb",
         "Histogram": "custom_pytorch_training.ipynb",
         "MixtureOfLogistics": "custom_pytorch_training.ipynb",
         "MNISTDataGenerator": "data_generator.ipynb",
         "KerasPipeline": "function_approximation.ipynb",
         "HighLevelKerasPipeline": "function_approximation.ipynb",
         "HighLevelKerasPipelineMultiTarget": "function_approximation.ipynb",
         "truncated": "gdrive_activity_tracking.ipynb",
         "getOneOf": "gdrive_activity_tracking.ipynb",
         "getTimeInfo": "gdrive_activity_tracking.ipynb",
         "getActionInfo": "gdrive_activity_tracking.ipynb",
         "getUserInfo": "gdrive_activity_tracking.ipynb",
         "getActorInfo": "gdrive_activity_tracking.ipynb",
         "getTargetInfo": "gdrive_activity_tracking.ipynb",
         "initialize_data": "gdrive_activity_tracking.ipynb",
         "get_authors": "gdrive_activity_tracking.ipynb",
         "convert_dates": "gdrive_activity_tracking.ipynb",
         "get_drive_activity": "gdrive_activity_tracking.ipynb",
         "SCOPES": "gdrive_activity_tracking.ipynb",
         "identify_actor": "gdrive_activity_tracking.ipynb",
         "check_data": "gdrive_activity_tracking.ipynb",
         "clean_data": "gdrive_activity_tracking.ipynb",
         "get_data": "gdrive_activity_tracking.ipynb",
         "MoneyAgent": "sample_mesa_model.ipynb",
         "MoneyModel": "sample_mesa_model.ipynb"}

modules = ["custom_pytorch_training.py",
           "custom_tf_training.py",
           "customizing_tf_fit.py",
           "data_generator.py",
           "dice_experiments.py",
           "function_approximation.py",
           "gdrive_activity_tracking.py",
           "sample_mesa.py"]

doc_url = "https://jovsa.github.io/jovsatools/"

git_url = "https://github.com/jovsa/jovsatools/tree/master/"

def custom_doc_links(name): return None
